@@ -16,8 +16,9 @@ class Channel(object):
 
         self.created:      Optional[DateTime] = None
 
-        self.list_modes:   Dict[str, List[str]]     = {}
-        self.modes:        Dict[str, Optional[str]] = {}
+        self._list_modes_temp: Dict[str, List[str]]     = {}
+        self.list_modes:       Dict[str, List[str]]     = {}
+        self.modes:            Dict[str, Optional[str]] = {}
 
     def __repr__(self) -> str:
         return f"Channel(name={self.name!r})"
@@ -36,10 +37,11 @@ class Channel(object):
             param: Optional[str],
             list_mode: bool):
         if list_mode:
-            if not char in self.list_modes:
-                self.list_modes[char] = []
-            if not param in self.list_modes[char]:
-                self.list_modes[char].append(param or "")
+            if param is not None:
+                if not char in self.list_modes:
+                    self.list_modes[char] = []
+                if not param in self.list_modes[char]:
+                    self.list_modes[char].append(param)
         else:
             self.modes[char] = param
 
@@ -47,7 +49,8 @@ class Channel(object):
             char: str,
             param: Optional[str]):
         if char in self.list_modes:
-            if param in self.list_modes[char]:
+            if (param is not None and
+                    param in self.list_modes[char]):
                 self.list_modes[char].remove(param)
                 if not self.list_modes[char]:
                     del self.list_modes[char]
