@@ -1,5 +1,6 @@
 from typing    import Callable, Dict, List, Optional, Set, Tuple
-from irctokens import build, Hostmask, Line, StatefulDecoder, StatefulEncoder
+from irctokens import Line, build, Hostmask, StatefulDecoder, StatefulEncoder
+from irctokens import hostmask as hostmask_
 from pendulum  import from_timestamp, now
 
 from .user         import User
@@ -20,7 +21,7 @@ class ServerException(Exception):
 class ServerDisconnectedException(ServerException):
     pass
 
-WHO_TYPE = "524" # randomly generated
+WHO_TYPE = "735" # randomly generated
 TYPE_EMIT = Optional[Emit]
 
 class Server(object):
@@ -33,7 +34,9 @@ class Server(object):
         self.hostname: Optional[str] = None
         self.realname: Optional[str] = None
         self.account:  Optional[str] = None
+        self.server:   Optional[str] = None
         self.away:     Optional[str] = None
+        self.ip:       Optional[str] = None
 
         self.registered = False
         self.modes: List[str] = []
@@ -54,15 +57,11 @@ class Server(object):
     def __repr__(self) -> str:
         return f"Server(name={self.name!r})"
 
-    def recv(self, data: bytes) -> List[Tuple[Line, TYPE_EMIT]]:
+    def recv(self, data: bytes) -> List[Line]:
         lines = self._decoder.push(data)
         if lines is None:
             raise ServerDisconnectedException()
-        emits: List[TYPE_EMIT] = []
-        for line in lines:
-            emit = self.parse_tokens(line)
-            emits.append(emit)
-        return list(zip(lines, emits))
+        return lines
 
     def parse_tokens(self, line: Line) -> TYPE_EMIT:
         ret_emit: TYPE_EMIT = None
@@ -110,7 +109,7 @@ class Server(object):
         return channel_user
 
     def prepare_whox(self, target: str) -> Line:
-        return build("WHO", [target, f"n%ahinrtu,{WHO_TYPE}"])
+        return build("WHO", [target, f"n%afhinrstu,{WHO_TYPE}"])
 
     def _self_hostmask(self, hostmask: Hostmask):
         self.nickname = hostmask.nickname
@@ -338,7 +337,7 @@ class Server(object):
                     else:
                         break
 
-                hostmask = Hostmask.from_source(nickname[len(modes):])
+                hostmask = hostmask_(nickname[len(modes):])
                 nickname_lower = self.casefold(hostmask.nickname)
                 if not nickname_lower in self.users:
                     self._add_user(hostmask.nickname, nickname_lower)
@@ -378,7 +377,7 @@ class Server(object):
             channel = self.channels[channel_lower]
             emit.channel = channel
             channel.topic        = line.params[1]
-            channel.topic_setter = str(line.hostmask)
+            channel.topic_setter = line.source
             channel.topic_time   = now("utc")
         return emit
 
@@ -428,23 +427,25 @@ class Server(object):
                             channel_user.modes.append(char)
                     elif char in channel_user.modes:
                         channel_user.modes.remove(char)
-            elif add:
+            else:
+                has_arg = False
+                is_list = False
                 if char in self.isupport.chanmodes.a_modes:
+                    has_arg = True
+                    is_list = True
+                elif add:
+                    has_arg = char in (self.isupport.chanmodes.b_modes+
+                        self.isupport.chanmodes.c_modes)
+                else: # remove
+                    has_arg = char in self.isupport.chanmodes.b_modes
+
+                if has_arg:
                     arg = params.pop(0)
-                    channel.add_mode(char, arg, True)
-                elif (char in self.isupport.chanmodes.b_modes or
-                        char in self.isupport.chanmodes.c_modes):
-                    arg = params.pop(0)
-                    channel.add_mode(char, arg, False)
+
+                if add:
+                    channel.add_mode(char, arg, is_list)
                 else:
-                    channel.add_mode(char, None, False)
-            else: # remove
-                if (char in self.isupport.chanmodes.a_modes or
-                        char in self.isupport.chanmodes.b_modes):
-                    arg = params.pop(0)
                     channel.remove_mode(char, arg)
-                else:
-                    channel.remove_mode(char, None)
 
             tokens.append((mode, arg))
 
@@ -514,6 +515,64 @@ class Server(object):
                 self.modes.append(char)
         return self._emit()
 
+    def _mode_list(self,
+            channel_name: str,
+            mode: str,
+            mask: str):
+        channel_lower = self.casefold(channel_name)
+        if channel_lower in self.channels:
+            channel = self.channels[channel_lower]
+            if not mode in channel._list_modes_temp:
+                channel._list_modes_temp[mode] = []
+            channel._list_modes_temp[mode].append(mask)
+    def _mode_list_end(self,
+            channel_name: str,
+            mode: str):
+        channel_lower = self.casefold(channel_name)
+        if channel_lower in self.channels:
+            channel = self.channels[channel_lower]
+            if mode in channel._list_modes_temp:
+                mlist = channel._list_modes_temp.pop(mode)
+                for mask in mlist:
+                    channel.add_mode(mode, mask, True)
+
+    @line_handler(RPL_BANLIST)
+    def _handle_banlist(self, line: Line) -> Emit:
+        channel = line.params[1]
+        mask    = line.params[2]
+
+        if len(line.params) > 3:
+            # parse these out but we're not storing them yet
+            set_by  = line.params[3]
+            set_at  = int(line.params[4])
+
+        self._mode_list(channel, "b", mask)
+        return self._emit()
+
+    @line_handler(RPL_ENDOFBANLIST)
+    def _handle_banlist_end(self, line: Line) -> Emit:
+        channel = line.params[1]
+        self._mode_list_end(channel, "b")
+        return self._emit()
+
+    @line_handler(RPL_QUIETLIST)
+    def _handle_quietlist(self, line: Line) -> Emit:
+        channel = line.params[1]
+        mode    = line.params[2]
+        mask    = line.params[3]
+        set_by  = line.params[4]
+        set_at  = int(line.params[5])
+
+        self._mode_list(channel, mode, mask)
+        return self._emit()
+
+    @line_handler(RPL_ENDOFQUIETLIST)
+    def _handle_quietlist_end(self, line: Line) -> Emit:
+        channel = line.params[1]
+        mode    = line.params[2]
+        self._mode_list_end(channel, mode)
+        return self._emit()
+
     @line_handler("PRIVMSG")
     @line_handler("NOTICE")
     @line_handler("TAGMSG")
@@ -577,14 +636,22 @@ class Server(object):
         nickname = line.params[5]
         username = line.params[2]
         hostname = line.params[3]
+        status   = line.params[6]
+        away     = "" if "G" in status else None
         realname = line.params[7].split(" ", 1)[1]
 
+        server:  Optional[str] = None
+        if not line.params[4] == "*":
+            server  = line.params[4]
+
         nickname_lower = self.casefold(line.params[5])
         if nickname_lower == self.nickname_lower:
             emit.self = True
             self.username = username
             self.hostname = hostname
             self.realname = realname
+            self.server   = server
+            self.away     = away
 
         if nickname_lower in self.users:
             user = self.users[nickname_lower]
@@ -592,21 +659,31 @@ class Server(object):
             user.username = username
             user.hostname = hostname
             user.realname = realname
+            user.server   = server
+            user.away     = away
         return emit
 
     @line_handler(RPL_WHOSPCRPL)
     # WHOX line, "WHO #channel|nickname" response; only listen for our "type"
     def _handle_whox(self, line: Line) -> Emit:
         emit = self._emit()
-        if line.params[1] == WHO_TYPE and len(line.params) == 8:
-            nickname_lower = self.casefold(line.params[5])
+        if line.params[1] == WHO_TYPE and len(line.params) == 10:
+            nickname_lower = self.casefold(line.params[6])
             username = line.params[2]
             hostname = line.params[4]
-            realname = line.params[7]
+            status   = line.params[7]
+            away     = "" if "G" in status else None
+            realname = line.params[9]
 
             account: Optional[str] = None
-            if not line.params[6] == "0":
-                account = line.params[6]
+            if not line.params[8] == "0":
+                account = line.params[8]
+            server:  Optional[str] = None
+            if not line.params[5] == "*":
+                server  = line.params[5]
+            ip:      Optional[str] = None
+            if not line.params[3] == "255.255.255.255":
+                ip = line.params[3]
 
             if nickname_lower in self.users:
                 user = self.users[nickname_lower]
@@ -615,12 +692,21 @@ class Server(object):
                 user.hostname = hostname
                 user.realname = realname
                 user.account  = account
+                user.server   = server
+                user.away     = away
+                if ip is not None:
+                    user.ip   = ip
+
             if nickname_lower == self.nickname_lower:
                 emit.self = True
                 self.username = username
                 self.hostname = hostname
                 self.realname = realname
                 self.account  = account
+                self.server   = server
+                self.away     = away
+                if ip is not None:
+                    self.ip   = ip
 
         return emit
 
@@ -681,6 +767,21 @@ class Server(object):
             user.realname = realname
         return emit
 
+    @line_handler(RPL_AWAY)
+    # sent in response to a command directed at a user who is marked as away
+    def _handle_RPL_AWAY(self, line: Line) -> Emit:
+        nickname       = line.params[1]
+        nickname_lower = self.casefold(nickname)
+        reason         = line.params[2]
+
+        if nickname_lower == self.nickname_lower:
+            self.away = reason
+        if nickname_lower in self.users:
+            user = self.users[nickname_lower]
+            user.away = reason
+
+        return self._emit()
+
     @line_handler("AWAY")
     def _handle_AWAY(self, line: Line) -> Emit:
         emit = self._emit()
@@ -758,7 +859,7 @@ class Server(object):
     @line_handler(RPL_LOGGEDIN)
     def _handle_loggedin(self, line: Line) -> Emit:
         hostmask_str = line.params[1]
-        hostmask     = Hostmask.from_source(hostmask_str)
+        hostmask     = hostmask_(hostmask_str)
         account      = line.params[2]
 
         self.account = account
@@ -768,7 +869,7 @@ class Server(object):
     @line_handler(RPL_LOGGEDOUT)
     def _handle_loggedout(self, line: Line) -> Emit:
         hostmask_str = line.params[1]
-        hostmask     = Hostmask.from_source(hostmask_str)
+        hostmask     = hostmask_(hostmask_str)
 
         self.account = None
         self._self_hostmask(hostmask)
